@@ -248,7 +248,7 @@ huffman_table_init_jpeg (JpegDecoder *decoder, HuffmanTable *table, JpegBits * b
 {
   int n_symbols;
   int huffsize[16];
-  int i, j, k;
+  int i, j;
   unsigned int symbol;
   int n = 0;
 
@@ -264,19 +264,23 @@ huffman_table_init_jpeg (JpegDecoder *decoder, HuffmanTable *table, JpegBits * b
     n_symbols += huffsize[i];
   }
 
+  /* There may be a maximum of 256 symbols in the table. */
+  if (n_symbols > 256) {
+    jpeg_decoder_error (decoder, "%u Huffman table entries exceeds 256", n_symbols);
+    return -1;
+  }
+
   /* Build up the symbol table.  The first symbol is all 0's, with
    * the number of bits determined by the first non-zero entry in
    * huffsize[].  Subsequent symbols with the same bit length are
    * incremented by 1.  Increasing the bit length shifts the
    * symbol 1 bit to the left. */
   symbol = 0;
-  k = 0;
   for (i = 0; i < 16; i++) {
     for (j = 0; j < huffsize[i]; j++) {
       huffman_table_add (table, symbol, i + 1, jpeg_bits_get_u8 (bits));
       n++;
       symbol++;
-      k++;
     }
     /* This checks that our symbol is actually less than the
      * number of bits we think it is.  This is only triggered
@@ -555,6 +559,8 @@ jpeg_decoder_free (JpegDecoder * dec)
 
   if (dec->data)
     free (dec->data);
+  if (dec->error_message)
+    free (dec->error_message);
 
   free (dec);
 }
