@@ -1,5 +1,5 @@
 /* Swfdec
- * Copyright (C) 2007 Benjamin Otte <otte@gnome.org>
+ * Copyright (C) 2007-2008 Benjamin Otte <otte@gnome.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -38,19 +38,17 @@
 #include "swfdec_as_types.h"
 #include "swfdec_constant_pool.h"
 #include "swfdec_debug.h"
+#include "swfdec_gc_object.h"
 #include "swfdec_internal.h" /* for swfdec_player_preinit_global() */
 #include "swfdec_script.h"
 
 /*** GARBAGE COLLECTION DOCS ***/
 
-#define SWFDEC_AS_GC_MARK (1 << 0)		/* only valid during GC */
-#define SWFDEC_AS_GC_ROOT (1 << 1)		/* for objects: rooted, for strings: static */
-
 /**
  * SECTION:Internals
- * @title: Internals and garbage collection
+ * @title: Internals of the script engine
  * @short_description: understanding internals such as garbage collection
- * @see_also: #SwfdecAsContext
+ * @see_also: #SwfdecAsContext, #SwfdecGcObject
  *
  * This section deals with the internals of the Swfdec Actionscript engine. You
  * should probably read this first when trying to write code with it. If you're
@@ -69,13 +67,10 @@
  * lookups.
  *
  * The script engine starts its life when it is initialized via 
- * swfdec_as_context_startup (). At that point, the basic objects are created.
- * After this function has been called, you can start executing code. All code
- * execution happens by creating a new #SwfdecAsFrame and then calling 
- * swfdec_as_context_run() to execute it. This function is the single entry 
- * point for code execution. Convenience functions exist that make executing 
- * code easy, most notably swfdec_as_object_run() and 
- * swfdec_as_object_call().
+ * swfdec_as_context_startup(). At that point, the basic objects are created.
+ * After this function has been called, you can start executing code. Code
+ * execution happens by calling swfdec_as_function_call_full() or convenience
+ * wrappers like swfdec_as_object_run() or swfdec_as_object_call().
  *
  * It is also easily possible to extend the environment by adding new objects.
  * In fact, without doing so, the environment is pretty bare as it just contains
@@ -169,8 +164,8 @@ swfdec_as_context_abort (SwfdecAsContext *context, const char *reason)
 {
   g_return_if_fail (context);
 
-  SWFDEC_ERROR ("%s", reason);
   if (context->state != SWFDEC_AS_CONTEXT_ABORTED) {
+    SWFDEC_ERROR ("abort: %s", reason);
     context->state = SWFDEC_AS_CONTEXT_ABORTED;
     g_object_notify (G_OBJECT (context), "aborted");
   }
@@ -215,17 +210,20 @@ swfdec_as_context_try_use_mem (SwfdecAsContext *context, gsize bytes)
  * keeps track of the memory that script code consumes. If too much memory is
  * in use, this function may decide to stop the script engine with an out of
  * memory error.
- *
- * Returns: %TRUE if the memory could be allocated. %FALSE on OOM.
  **/
-gboolean
+void
 swfdec_as_context_use_mem (SwfdecAsContext *context, gsize bytes)
 {
-  g_return_val_if_fail (SWFDEC_IS_AS_CONTEXT (context), FALSE);
-  g_return_val_if_fail (bytes > 0, FALSE);
+  g_return_if_fail (SWFDEC_IS_AS_CONTEXT (context));
+  g_return_if_fail (bytes > 0);
 
   /* FIXME: Don't forget to abort on OOM */
-  return swfdec_as_context_try_use_mem (context, bytes);
+  if (!swfdec_as_context_try_use_mem (context, bytes)) {
+    swfdec_as_context_abort (context, "Out of memory");
+    /* add the memory anyway, as we're gonna make use of it. */
+    context->memory += bytes;
+    context->memory_since_gc += bytes;
+  }
 }
 
 /**
@@ -278,23 +276,18 @@ swfdec_as_context_remove_strings (gpointer key, gpointer value, gpointer data)
 static gboolean
 swfdec_as_context_remove_objects (gpointer key, gpointer value, gpointer debugger)
 {
-  SwfdecAsObject *object;
+  SwfdecGcObject *gc;
 
-  object = key;
+  gc = key;
   /* we only check for mark here, not root, since this works on destroy, too */
-  if (object->flags & SWFDEC_AS_GC_MARK) {
-    object->flags &= ~SWFDEC_AS_GC_MARK;
-    SWFDEC_LOG ("%s: %s %p", (object->flags & SWFDEC_AS_GC_ROOT) ? "rooted" : "marked",
-	G_OBJECT_TYPE_NAME (object), object);
+  if (gc->flags & SWFDEC_AS_GC_MARK) {
+    gc->flags &= ~SWFDEC_AS_GC_MARK;
+    SWFDEC_LOG ("%s: %s %p", (gc->flags & SWFDEC_AS_GC_ROOT) ? "rooted" : "marked",
+	G_OBJECT_TYPE_NAME (gc), gc);
     return FALSE;
   } else {
-    SWFDEC_LOG ("deleted: %s %p", G_OBJECT_TYPE_NAME (object), object);
-    if (debugger) {
-      SwfdecAsDebuggerClass *klass = SWFDEC_AS_DEBUGGER_GET_CLASS (debugger);
-      if (klass->remove)
-	klass->remove (debugger, object->context, object);
-    }
-    swfdec_as_object_collect (object);
+    SWFDEC_LOG ("deleted: %s %p", G_OBJECT_TYPE_NAME (gc), gc);
+    g_object_unref (gc);
     return TRUE;
   }
 }
@@ -311,28 +304,6 @@ swfdec_as_context_collect (SwfdecAsContext *context)
   SWFDEC_INFO (">> done collecting garbage");
 }
 
-/**
- * swfdec_as_object_mark:
- * @object: a #SwfdecAsObject
- *
- * Mark @object as being in use. Calling this function is only valid during
- * the marking phase of garbage collection.
- **/
-void
-swfdec_as_object_mark (SwfdecAsObject *object)
-{
-  SwfdecAsObjectClass *klass;
-
-  g_return_if_fail (SWFDEC_IS_AS_OBJECT (object));
-
-  if (object->flags & SWFDEC_AS_GC_MARK)
-    return;
-  object->flags |= SWFDEC_AS_GC_MARK;
-  klass = SWFDEC_AS_OBJECT_GET_CLASS (object);
-  g_assert (klass->mark);
-  klass->mark (object);
-}
-
 /**
  * swfdec_as_string_mark:
  * @string: a garbage-collected string
@@ -366,7 +337,7 @@ swfdec_as_value_mark (SwfdecAsValue *value)
   g_return_if_fail (SWFDEC_IS_AS_VALUE (value));
 
   if (SWFDEC_AS_VALUE_IS_OBJECT (value)) {
-    swfdec_as_object_mark (SWFDEC_AS_VALUE_GET_OBJECT (value));
+    swfdec_gc_object_mark (value->value.object);
   } else if (SWFDEC_AS_VALUE_IS_STRING (value)) {
     swfdec_as_string_mark (SWFDEC_AS_VALUE_GET_STRING (value));
   }
@@ -375,10 +346,10 @@ swfdec_as_value_mark (SwfdecAsValue *value)
 static void
 swfdec_as_context_mark_roots (gpointer key, gpointer value, gpointer data)
 {
-  SwfdecAsObject *object = key;
+  SwfdecGcObject *object = key;
 
   if ((object->flags & (SWFDEC_AS_GC_MARK | SWFDEC_AS_GC_ROOT)) == SWFDEC_AS_GC_ROOT)
-    swfdec_as_object_mark (object);
+    swfdec_gc_object_mark (object);
 }
 
 /* FIXME: replace this with refcounted strings? */
@@ -389,7 +360,9 @@ swfdec_as_context_mark_constant_pools (gpointer key, gpointer value, gpointer un
   guint i;
 
   for (i = 0; i < swfdec_constant_pool_size (pool); i++) {
-    swfdec_as_string_mark (swfdec_constant_pool_get (pool, i));
+    const char *s = swfdec_constant_pool_get (pool, i);
+    if (s)
+      swfdec_as_string_mark (s);
   }
 }
 
@@ -398,11 +371,11 @@ swfdec_as_context_do_mark (SwfdecAsContext *context)
 {
   /* This if is needed for SwfdecPlayer */
   if (context->global) {
-    swfdec_as_object_mark (context->global);
-    swfdec_as_object_mark (context->Function);
-    swfdec_as_object_mark (context->Function_prototype);
-    swfdec_as_object_mark (context->Object);
-    swfdec_as_object_mark (context->Object_prototype);
+    swfdec_gc_object_mark (context->global);
+    swfdec_gc_object_mark (context->Function);
+    swfdec_gc_object_mark (context->Function_prototype);
+    swfdec_gc_object_mark (context->Object);
+    swfdec_gc_object_mark (context->Object_prototype);
   }
   if (context->exception)
     swfdec_as_value_mark (&context->exception_value);
@@ -619,8 +592,10 @@ swfdec_as_context_create_string (SwfdecAsContext *context, const char *string, g
 {
   char *new;
   
-  if (!swfdec_as_context_use_mem (context, sizeof (char) * (2 + len)))
+  if (!swfdec_as_context_try_use_mem (context, sizeof (char) * (2 + len))) {
+    swfdec_as_context_abort (context, "Out of memory");
     return SWFDEC_AS_STR_EMPTY;
+  }
 
   new = g_slice_alloc (2 + len);
   memcpy (&new[1], string, len);
@@ -811,7 +786,7 @@ swfdec_as_context_get_time (SwfdecAsContext *context, GTimeVal *tv)
 void
 swfdec_as_context_run (SwfdecAsContext *context)
 {
-  SwfdecAsFrame *frame, *last_frame;
+  SwfdecAsFrame *frame;
   SwfdecScript *script;
   const SwfdecActionSpec *spec;
   const guint8 *startpc, *pc, *endpc, *nextpc, *exitpc;
@@ -825,9 +800,24 @@ swfdec_as_context_run (SwfdecAsContext *context)
   gboolean check_block; /* some opcodes avoid a scope check */
 
   g_return_if_fail (SWFDEC_IS_AS_CONTEXT (context));
+  g_return_if_fail (context->frame != NULL);
+  g_return_if_fail (context->frame->script != NULL);
+  g_return_if_fail (context->global); /* check here because of swfdec_sandbox_(un)use() */
 
-  if (context->frame == NULL || context->state == SWFDEC_AS_CONTEXT_ABORTED)
-    return;
+  /* setup data */
+  frame = context->frame;
+  original_version = context->version;
+
+  /* sanity checks */
+  if (context->state == SWFDEC_AS_CONTEXT_ABORTED)
+    goto error;
+  if (!swfdec_as_context_check_continue (context))
+    goto error;
+  if (context->call_depth > 256) {
+    /* we've exceeded our maximum call depth, throw an error and abort */
+    swfdec_as_context_abort (context, "Stack overflow");
+    goto error;
+  }
 
   if (context->debugger) {
     SwfdecAsDebuggerClass *klass = SWFDEC_AS_DEBUGGER_GET_CLASS (context->debugger);
@@ -836,63 +826,6 @@ swfdec_as_context_run (SwfdecAsContext *context)
     step = NULL;
   }
 
-  last_frame = context->last_frame;
-  context->last_frame = context->frame->next;
-  original_version = context->version;
-start:
-  g_return_if_fail (context->global); /* check here because of swfdec_sandbox_(un)use() */
-  if (!swfdec_as_context_check_continue (context))
-    goto error;
-  /* setup data */
-  frame = context->frame;
-  if (frame == context->last_frame)
-    goto out;
-  if (context->call_depth > 256) {
-    /* we've exceeded our maximum call depth, throw an error and abort */
-    swfdec_as_context_abort (context, "Stack overflow");
-    goto error;
-  }
-  if (SWFDEC_IS_AS_NATIVE_FUNCTION (frame->function)) {
-    SwfdecAsNativeFunction *native = SWFDEC_AS_NATIVE_FUNCTION (frame->function);
-    SwfdecAsValue rval = { 0, };
-    if (frame->argc >= native->min_args && 
-	(native->type == 0 || 
-	 g_type_is_a (G_OBJECT_TYPE (frame->thisp), native->type))) {
-      SwfdecAsValue *argv;
-      /* accumulate argv */
-      if (frame->argc == 0 || frame->argv != NULL) {
-	/* FIXME FIXME FIXME: no casting here please! */
-	argv = (SwfdecAsValue *) frame->argv;
-      } else {
-	SwfdecAsStack *stack;
-	SwfdecAsValue *cur;
-	guint i;
-	if (frame->argc > 128) {
-	  SWFDEC_FIXME ("allow calling native functions with more than 128 args (this one has %u)",
-	      frame->argc);
-	  frame->argc = 128;
-	}
-	argv = g_new (SwfdecAsValue, frame->argc);
-	stack = context->stack;
-	cur = context->cur;
-	for (i = 0; i < frame->argc; i++) {
-	  if (cur <= &stack->elements[0]) {
-	    stack = stack->next;
-	    cur = &stack->elements[stack->used_elements];
-	  }
-	  cur--;
-	  argv[i] = *cur;
-	}
-      }
-      native->native (context, frame->thisp, frame->argc, 
-	  argv, &rval);
-      if (argv != frame->argv)
-	g_free (argv);
-    }
-    swfdec_as_frame_return (frame, &rval);
-    goto start;
-  }
-  g_assert (frame->script);
   g_assert (frame->target);
   script = frame->script;
   context->version = script->version;
@@ -906,13 +839,13 @@ start:
     if (context->exception) {
       swfdec_as_frame_handle_exception (frame);
       if (frame != context->frame)
-	goto start;
+	goto out;
       pc = frame->pc;
       continue;
     }
     if (pc == exitpc) {
       swfdec_as_frame_return (frame, NULL);
-      goto start;
+      goto out;
     }
     if (pc < startpc || pc >= endpc) {
       SWFDEC_ERROR ("pc %p not in valid range [%p, %p) anymore", pc, startpc, endpc);
@@ -920,10 +853,10 @@ start:
     }
     while (check_block && (pc < frame->block_start || pc >= frame->block_end)) {
       SWFDEC_LOG ("code exited block");
-      swfdec_as_frame_pop_block (frame);
+      swfdec_as_frame_pop_block (frame, context);
       pc = frame->pc;
       if (frame != context->frame)
-	goto start;
+	goto out;
       if (context->exception)
 	break;
     }
@@ -932,18 +865,14 @@ start:
 
     /* decode next action */
     action = *pc;
-    if (action == 0) {
-      swfdec_as_frame_return (frame, NULL);
-      goto start;
-    }
     /* invoke debugger if there is one */
     if (step) {
       frame->pc = pc;
       (* step) (context->debugger, context);
-      if (frame != context->frame || 
-	  frame->pc != pc) {
-	goto start;
-      }
+      if (frame != context->frame)
+	goto out;
+      if (frame->pc != pc)
+	continue;
     }
     /* prepare action */
     spec = swfdec_as_actions + action;
@@ -1016,196 +945,18 @@ start:
 #endif
     } else {
       /* someone called/returned from a function, reread variables */
-      goto start;
+      goto out;
     }
   }
 
 error:
-  while (context->frame != context->last_frame)
-    swfdec_as_frame_return (context->frame, NULL);
+  if (context->frame == frame)
+    swfdec_as_frame_return (frame, NULL);
 out:
-  context->last_frame = last_frame;
   context->version = original_version;
   return;
 }
 
-/*** EVAL ***/
-
-static char *
-swfdec_as_slash_to_dot (const char *slash_str)
-{
-  const char *cur = slash_str;
-  GString *str = g_string_new ("");
-
-  if (*cur == '/') {
-    g_string_append (str, "_root");
-  } else {
-    goto start;
-  }
-  while (cur && *cur == '/') {
-    cur++;
-start:
-    if (str->len > 0)
-      g_string_append_c (str, '.');
-    if (cur[0] == '.' && cur[1] == '.') {
-      g_string_append (str, "_parent");
-      cur += 2;
-    } else {
-      char *slash = strchr (cur, '/');
-      if (slash) {
-	g_string_append_len (str, cur, slash - cur);
-	cur = slash;
-      } else {
-	g_string_append (str, cur);
-	cur = NULL;
-      }
-    }
-    /* cur should now point to the slash */
-  }
-  if (cur) {
-    if (*cur != '\0')
-      goto fail;
-  }
-  SWFDEC_DEBUG ("parsed slash-notated string \"%s\" into dot notation \"%s\"",
-      slash_str, str->str);
-  return g_string_free (str, FALSE);
-
-fail:
-  SWFDEC_WARNING ("failed to parse slash-notated string \"%s\" into dot notation", slash_str);
-  g_string_free (str, TRUE);
-  return NULL;
-}
-
-static void
-swfdec_as_context_eval_get_property (SwfdecAsContext *cx, 
-    SwfdecAsObject *obj, const char *name, SwfdecAsValue *ret)
-{
-  if (obj) {
-    swfdec_as_object_get_variable (obj, name, ret);
-  } else {
-    if (cx->frame) {
-      swfdec_as_frame_get_variable (cx->frame, name, ret);
-    } else {
-      SWFDEC_WARNING ("eval called without a frame");
-      swfdec_as_object_get_variable (cx->global, name, ret);
-    }
-  }
-}
-
-static void
-swfdec_as_context_eval_set_property (SwfdecAsContext *cx, 
-    SwfdecAsObject *obj, const char *name, const SwfdecAsValue *ret)
-{
-  if (obj == NULL) {
-    if (cx->frame == NULL) {
-      SWFDEC_ERROR ("no frame in eval_set?");
-      return;
-    }
-    swfdec_as_frame_set_variable (cx->frame, name, ret, TRUE, FALSE);
-  } else {
-    swfdec_as_object_set_variable (obj, name, ret);
-  }
-}
-
-static void
-swfdec_as_context_eval_internal (SwfdecAsContext *cx, SwfdecAsObject *obj, const char *str,
-        SwfdecAsValue *val, gboolean set)
-{
-  SwfdecAsValue cur;
-  char **varlist;
-  guint i;
-
-  SWFDEC_LOG ("eval called with \"%s\" on %p", str, obj);
-  if (strchr (str, '/')) {
-    char *work = swfdec_as_slash_to_dot (str);
-    if (!work) {
-      SWFDEC_AS_VALUE_SET_UNDEFINED (val);
-      return;
-    }
-    varlist = g_strsplit (work, ".", -1);
-    g_free (work);
-  } else {
-    varlist = g_strsplit (str, ".", -1);
-  }
-  for (i = 0; varlist[i] != NULL; i++) {
-    const char *dot = swfdec_as_context_get_string (cx, varlist[i]);
-    if (varlist[i+1] != NULL) {
-      swfdec_as_context_eval_get_property (cx, obj, dot, &cur);
-      if (!SWFDEC_AS_VALUE_IS_OBJECT (&cur)) {
-	SWFDEC_AS_VALUE_SET_UNDEFINED (&cur);
-	break;
-      }
-      obj = SWFDEC_AS_VALUE_GET_OBJECT (&cur);
-    } else {
-      if (set) {
-	swfdec_as_context_eval_set_property (cx, obj, dot, val);
-      } else {
-	swfdec_as_context_eval_get_property (cx, obj, dot, &cur);
-      }
-      goto finish;
-    }
-  }
-  if (obj == NULL) {
-    if (cx->frame)
-      obj = cx->frame->target;
-    else
-      obj = cx->global;
-  }
-  g_assert (obj != NULL);
-  SWFDEC_AS_VALUE_SET_OBJECT (&cur, obj);
-
-finish:
-  g_strfreev (varlist);
-  *val = cur;
-}
-
-/**
- * swfdec_as_context_eval:
- * @context: a #SwfdecAsContext
- * @obj: #SwfdecAsObject to use as source for evaluating or NULL for the 
- *       current frame's scope
- * @str: The string to evaluate
- * @val: location for the return value
- *
- * This function works like the Actionscript eval function used on @obj.
- * It handles both slash-style and dot-style notation. If an error occured
- * during evaluation, the return value will be the undefined value.
- **/
-void
-swfdec_as_context_eval (SwfdecAsContext *context, SwfdecAsObject *obj, const char *str, 
-    SwfdecAsValue *val)
-{
-  g_return_if_fail (SWFDEC_IS_AS_CONTEXT (context));
-  g_return_if_fail (obj == NULL || SWFDEC_IS_AS_OBJECT (obj));
-  g_return_if_fail (str != NULL);
-  g_return_if_fail (val != NULL);
-
-  swfdec_as_context_eval_internal (context, obj, str, val, FALSE);
-}
-
-/**
- * swfdec_as_context_eval_set:
- * @context: a #SwfdecAsContext
- * @obj: #SwfdecAsObject to use as source for evaluating or NULL for the 
- *       default object.
- * @str: The string to evaluate
- * @val: the value to set the variable to
- *
- * Sets the variable referenced by @str to @val. If @str does not reference 
- * a valid property, nothing happens.
- **/
-void
-swfdec_as_context_eval_set (SwfdecAsContext *context, SwfdecAsObject *obj, const char *str,
-    const SwfdecAsValue *val)
-{
-  g_return_if_fail (SWFDEC_IS_AS_CONTEXT (context));
-  g_return_if_fail (obj == NULL || SWFDEC_IS_AS_OBJECT (obj));
-  g_return_if_fail (str != NULL);
-  g_return_if_fail (val != NULL);
-
-  swfdec_as_context_eval_internal (context, obj, str, (SwfdecAsValue *) val, TRUE);
-}
-
 /*** AS CODE ***/
 
 static void
@@ -1302,34 +1053,14 @@ swfdec_as_context_parseInt (SwfdecAsContext *cx, SwfdecAsObject *object,
 {
   const char *s;
   char *tail;
-  int radix;
+  int radix = 10;
   gint64 i;
 
-  if (argc < 1)
+  SWFDEC_AS_CHECK (0, NULL, "s|i", &s, &radix);
+
+  if (argc >= 2 && (radix < 2 || radix > 36)) {
+    SWFDEC_AS_VALUE_SET_NUMBER (retval, NAN);
     return;
-
-  s = swfdec_as_value_to_string (cx, &argv[0]);
-
-  if (argc >= 2) {
-    radix = swfdec_as_value_to_integer (cx, &argv[1]);
-
-    if (radix < 2 || radix > 36) {
-      SWFDEC_AS_VALUE_SET_NUMBER (retval, NAN);
-      return;
-    }
-
-    // special case, strtol parses things that we shouldn't parse
-    if (radix == 16) {
-      const char *end = s + strspn (s, " \t\r\n");
-      if (end != s && (end[0] == '-' || end[0] == '+'))
-	end++;
-      if (end != s && end[0] == '0' && (end[1] == 'x' || end[1] == 'X')) {
-	SWFDEC_AS_VALUE_SET_NUMBER (retval, 0);
-	return;
-      }
-    }
-  } else {
-    radix = 0;
   }
 
   // special case, don't allow sign in front of the 0x
@@ -1339,15 +1070,35 @@ swfdec_as_context_parseInt (SwfdecAsContext *cx, SwfdecAsObject *object,
     return;
   }
 
-  if (s[0] == '0' && (s[1] == 'x' || s[1] == 'X')) {
-    s = s + 2;
-    i = g_ascii_strtoll (s, &tail, (radix != 0 ? radix : 16));
-  } else if (s[0] == '0' && s[strspn (s, "01234567")] == '\0') {
-    i = g_ascii_strtoll (s, &tail, (radix != 0 ? radix : 8));
-  } else {
-    i = g_ascii_strtoll (s, &tail, (radix != 0 ? radix : 10));
+  // automatic radix
+  if (argc < 2) {
+    if (s[0] == '0' && (s[1] == 'x' || s[1] == 'X')) {
+      radix = 16;
+    } else if ((s[0] == '0' || ((s[0] == '+' || s[0] == '-') && s[1] == '0')) &&
+	s[strspn (s+1, "01234567") + 1] == '\0') {
+      radix = 8;
+    } else {
+      radix = 10;
+    }
   }
 
+  // skip 0x at the start
+  if (s[0] == '0' && (s[1] == 'x' || s[1] == 'X'))
+    s += 2;
+
+  // strtoll parses strings with 0x when given radix 16, but we don't want that
+  if (radix == 16) {
+    const char *skip = s + strspn (s, " \t\r\n");
+    if (skip != s && (skip[0] == '-' || skip[0] == '+'))
+      skip++;
+    if (skip != s && skip[0] == '0' && (skip[1] == 'x' || skip[1] == 'X')) {
+      SWFDEC_AS_VALUE_SET_NUMBER (retval, 0);
+      return;
+    }
+  }
+
+  i = g_ascii_strtoll (s, &tail, radix);
+
   if (tail == s) {
     SWFDEC_AS_VALUE_SET_NUMBER (retval, NAN);
     return;
