@@ -23,10 +23,8 @@
 
 #include "swfdec_as_function.h"
 #include "swfdec_as_context.h"
-#include "swfdec_as_frame_internal.h"
 #include "swfdec_as_internal.h"
-#include "swfdec_as_stack.h"
-#include "swfdec_as_super.h"
+#include "swfdec_as_native_function.h"
 #include "swfdec_as_strings.h"
 #include "swfdec_debug.h"
 
@@ -98,74 +96,46 @@ swfdec_as_function_set_constructor (SwfdecAsFunction *fun)
       SWFDEC_AS_VARIABLE_VERSION_6_UP);
 }
 
-SwfdecAsFrame *
-swfdec_as_function_call_no_preload (SwfdecAsFunction *function, 
-    SwfdecAsObject *thisp, guint n_args, const SwfdecAsValue *args, 
-    SwfdecAsValue *return_value)
-{
-  SwfdecAsFrame *frame;
-  SwfdecAsFunctionClass *klass;
-
-  g_return_val_if_fail (SWFDEC_IS_AS_FUNCTION (function), NULL);
-  g_return_val_if_fail (thisp == NULL || SWFDEC_IS_AS_OBJECT (thisp), NULL);
-
-  /* just to be sure... */
-  if (return_value)
-    SWFDEC_AS_VALUE_SET_UNDEFINED (return_value);
-
-  klass = SWFDEC_AS_FUNCTION_GET_CLASS (function);
-  g_assert (klass->call);
-  frame = klass->call (function);
-  /* FIXME: figure out what to do in these situations?
-   * It's a problem when called inside swfdec_as_function_call () as the
-   * user of that function expects success, but super may fail here */
-  if (frame == NULL)
-    return NULL;
-  /* second check especially for super object */
-  if (thisp != NULL && frame->thisp == NULL) {
-    swfdec_as_frame_set_this (frame, swfdec_as_object_resolve (thisp));
-  }
-  frame->is_local = TRUE;
-  frame->argc = n_args;
-  frame->argv = args;
-  frame->return_value = return_value;
-  return frame;
-}
-
 /**
  * swfdec_as_function_call:
  * @function: the #SwfdecAsFunction to call
  * @thisp: this argument to use for the call or %NULL for none
  * @n_args: number of arguments to pass to the function
  * @args: the arguments to pass or %NULL to read the last @n_args stack elements.
- *        The memory must be unchanged until the function call has completed.
- *        This is after the call to swfdec_as_context_run () has finished.
  * @return_value: pointer for return value or %NULL to push the return value to 
  *                the stack
  *
- * Calls the given function. This means a #SwfdecAsFrame is created for the 
- * function and pushed on top of the execution stack. The function is however
- * not executed. Call swfdec_as_context_run () to execute it.
+ * Calls the given function. This is a macro that resolves to 
+ * swfdec_as_function_call_full().
+ **/
+/**
+ * swfdec_as_function_call_full:
+ * @function: the #SwfdecAsFunction to call
+ * @thisp: this argument to use for the call or %NULL for none
+ * @construct: call this function as a constructor. This is only relevant for 
+ *             native functions.
+ * @super_reference: The object to be referenced by the super object in this 
+ *                   function call or %NULL to use the default.
+ * @n_args: number of arguments to pass to the function
+ * @args: the arguments to pass or %NULL to read the last @n_args stack elements.
+ * @return_value: pointer for return value or %NULL to push the return value to 
+ *                the stack
+ *
+ * Calls the given function.
  **/
 void
-swfdec_as_function_call (SwfdecAsFunction *function, SwfdecAsObject *thisp, guint n_args,
+swfdec_as_function_call_full (SwfdecAsFunction *function, SwfdecAsObject *thisp, 
+    gboolean construct, SwfdecAsObject *super_reference, guint n_args, 
     const SwfdecAsValue *args, SwfdecAsValue *return_value)
 {
-  SwfdecAsFrame *frame;
+  SwfdecAsFunctionClass *klass;
 
   g_return_if_fail (SWFDEC_IS_AS_FUNCTION (function));
   g_return_if_fail (thisp == NULL || SWFDEC_IS_AS_OBJECT (thisp));
+  g_return_if_fail (super_reference == NULL || SWFDEC_IS_AS_OBJECT (super_reference));
 
-  frame = swfdec_as_function_call_no_preload (function, thisp, n_args, args, return_value);
-  if (frame == NULL)
-    return;
-  if (thisp != NULL) {
-    swfdec_as_super_new (frame, thisp, thisp->prototype);
-  } else {
-    // FIXME: Does the super object really reference the function when thisp is NULL?
-    swfdec_as_super_new (frame, SWFDEC_AS_OBJECT (function), SWFDEC_AS_OBJECT (function)->prototype);
-  }
-  swfdec_as_frame_preload (frame);
+  klass = SWFDEC_AS_FUNCTION_GET_CLASS (function);
+  klass->call (function, thisp, construct, super_reference, n_args, args, return_value);
 }
 
 /*** AS CODE ***/
@@ -176,7 +146,7 @@ swfdec_as_function_do_call (SwfdecAsContext *cx, SwfdecAsObject *object,
     guint argc, SwfdecAsValue *argv, SwfdecAsValue *ret)
 {
   SwfdecAsFunction *fun;
-  SwfdecAsObject *thisp;
+  SwfdecAsObject *thisp = NULL;
 
   SWFDEC_AS_CHECK (SWFDEC_TYPE_AS_FUNCTION, &fun, "|O", &thisp);
 
@@ -188,7 +158,6 @@ swfdec_as_function_do_call (SwfdecAsContext *cx, SwfdecAsObject *object,
     argv++;
   }
   swfdec_as_function_call (fun, thisp, argc, argv, ret);
-  swfdec_as_context_run (cx);
 }
 
 SWFDEC_AS_NATIVE (101, 11, swfdec_as_function_apply)
@@ -199,7 +168,7 @@ swfdec_as_function_apply (SwfdecAsContext *cx, SwfdecAsObject *object,
   SwfdecAsValue *argv_pass = NULL;
   int length = 0;
   SwfdecAsFunction *fun;
-  SwfdecAsObject *thisp;
+  SwfdecAsObject *thisp = NULL;
 
   SWFDEC_AS_CHECK (SWFDEC_TYPE_AS_FUNCTION, &fun, "|O", &thisp);
 
@@ -234,7 +203,6 @@ swfdec_as_function_apply (SwfdecAsContext *cx, SwfdecAsObject *object,
   }
 
   swfdec_as_function_call (fun, thisp, length, argv_pass, ret);
-  swfdec_as_context_run (cx);
 
   if (argv_pass) {
     swfdec_as_context_unuse_mem (cx, sizeof (SwfdecAsValue) * length);
@@ -251,7 +219,7 @@ swfdec_as_function_init_context (SwfdecAsContext *context)
   g_return_if_fail (SWFDEC_IS_AS_CONTEXT (context));
 
   function = SWFDEC_AS_OBJECT (swfdec_as_object_add_function (context->global,
-      SWFDEC_AS_STR_Function, 0, NULL, 0));
+      SWFDEC_AS_STR_Function, NULL));
   swfdec_as_object_set_variable_flags (context->global, SWFDEC_AS_STR_Function, SWFDEC_AS_VARIABLE_VERSION_6_UP);
   context->Function = function;
   SWFDEC_AS_VALUE_SET_OBJECT (&val, function);
