@@ -52,30 +52,72 @@
 
 G_DEFINE_TYPE (SwfdecAsNativeFunction, swfdec_as_native_function, SWFDEC_TYPE_AS_FUNCTION)
 
-static SwfdecAsFrame *
-swfdec_as_native_function_call (SwfdecAsFunction *function)
+static void
+swfdec_as_native_function_call (SwfdecAsFunction *function, SwfdecAsObject *thisp, 
+    gboolean construct, SwfdecAsObject *super_reference, guint n_args, 
+    const SwfdecAsValue *args, SwfdecAsValue *return_value)
 {
   SwfdecAsNativeFunction *native = SWFDEC_AS_NATIVE_FUNCTION (function);
-  SwfdecAsFrame *frame;
-  SwfdecAsContext *cx;
+  SwfdecAsContext *cx = swfdec_gc_object_get_context (function);
+  SwfdecAsFrame frame = { NULL, };
+  SwfdecAsValue rval = { 0, };
+  SwfdecAsValue *argv;
+
+  if (construct && native->construct_type != 0 &&
+      !g_type_is_a (G_OBJECT_TYPE (thisp), native->construct_type)) {
+    SWFDEC_FIXME ("Swfdec can't handle call to native constructor with invalid type");
+    return;
+  }
 
-  cx = SWFDEC_AS_OBJECT (function)->context;
-  frame = swfdec_as_frame_new_native (cx);
-  if (frame == NULL)
-    return NULL;
   g_assert (native->name);
-  frame->function_name = native->name;
-  frame->function = function;
+
+  swfdec_as_frame_init_native (&frame, cx);
+  frame.construct = construct;
+  frame.function = function;
   /* We copy the target here so we have a proper SwfdecMovie reference inside native 
    * functions. This is for example necessary for swfdec_player_get_movie_by_value()
    * and probably other stuff that does variable lookups inside native functions.
    */
   /* FIXME: copy target or original target? */
-  if (frame->next) {
-    frame->target = frame->next->original_target;
-    frame->original_target = frame->target;
+  if (frame.next) {
+    frame.target = frame.next->original_target;
+    frame.original_target = frame.target;
   }
-  return frame;
+  if (thisp)
+    swfdec_as_frame_set_this (&frame, swfdec_as_object_resolve (thisp));
+  frame.argc = n_args;
+  frame.argv = args;
+  frame.return_value = return_value;
+  frame.construct = construct;
+
+  if (frame.argc == 0 || frame.argv != NULL) {
+    /* FIXME FIXME FIXME: no casting here please! */
+    argv = (SwfdecAsValue *) frame.argv;
+  } else {
+    SwfdecAsStack *stack;
+    SwfdecAsValue *cur;
+    guint i;
+    if (frame.argc > 128) {
+      SWFDEC_FIXME ("allow calling native functions with more than 128 args (this one has %u)",
+	  frame.argc);
+      frame.argc = 128;
+    }
+    argv = g_new (SwfdecAsValue, frame.argc);
+    stack = cx->stack;
+    cur = cx->cur;
+    for (i = 0; i < frame.argc; i++) {
+      if (cur <= &stack->elements[0]) {
+	stack = stack->next;
+	cur = &stack->elements[stack->used_elements];
+      }
+      cur--;
+      argv[i] = *cur;
+    }
+  }
+  native->native (cx, frame.thisp, frame.argc, argv, &rval);
+  if (argv != frame.argv)
+    g_free (argv);
+  swfdec_as_frame_return (&frame, &rval);
 }
 
 static char *
@@ -121,21 +163,17 @@ swfdec_as_native_function_init (SwfdecAsNativeFunction *function)
  * @context: a #SwfdecAsContext
  * @name: name of the function
  * @native: function to call when executed
- * @min_args: minimum number of arguments required
  * @prototype: The object to be used as "prototype" property for the created 
  *             function or %NULL for none.
  *
- * Creates a new native function, that will execute @native when called. The
- * @min_args parameter sets a requirement for the minimum number of arguments
- * to pass to @native. If the function gets called with less arguments, it
- * will just redurn undefined. You might want to use 
- * swfdec_as_object_add_function() instead of this function.
+ * Creates a new native function, that will execute @native when called. You 
+ * might want to use swfdec_as_object_add_function() instead of this function.
  *
- * Returns: a new #SwfdecAsFunction or %NULL on OOM
+ * Returns: a new #SwfdecAsFunction
  **/
 SwfdecAsFunction *
 swfdec_as_native_function_new (SwfdecAsContext *context, const char *name,
-    SwfdecAsNative native, guint min_args, SwfdecAsObject *prototype)
+    SwfdecAsNative native, SwfdecAsObject *prototype)
 {
   SwfdecAsNativeFunction *fun;
 
@@ -143,15 +181,9 @@ swfdec_as_native_function_new (SwfdecAsContext *context, const char *name,
   g_return_val_if_fail (native != NULL, NULL);
   g_return_val_if_fail (prototype == NULL || SWFDEC_IS_AS_OBJECT (prototype), NULL);
 
-  if (!swfdec_as_context_use_mem (context, sizeof (SwfdecAsNativeFunction)))
-    return NULL;
-  fun = g_object_new (SWFDEC_TYPE_AS_NATIVE_FUNCTION, NULL);
-  if (fun == NULL)
-    return NULL;
+  fun = g_object_new (SWFDEC_TYPE_AS_NATIVE_FUNCTION, "context", context, NULL);
   fun->native = native;
-  fun->min_args = min_args;
   fun->name = g_strdup (name);
-  swfdec_as_object_add (SWFDEC_AS_OBJECT (fun), context, sizeof (SwfdecAsNativeFunction));
   /* need to set prototype before setting the constructor or Function.constructor 
    * being CONSTANT disallows setting it. */
   if (prototype) {
@@ -165,27 +197,6 @@ swfdec_as_native_function_new (SwfdecAsContext *context, const char *name,
   return SWFDEC_AS_FUNCTION (fun);
 }
 
-/**
- * swfdec_as_native_function_set_object_type:
- * @function: a #SwfdecAsNativeFunction
- * @type: required #GType for the this object
- *
- * Sets the required type for the this object to @type. If the this object 
- * isn't of the required type, the function will not be called and its
- * return value will be undefined.
- **/
-void
-swfdec_as_native_function_set_object_type (SwfdecAsNativeFunction *function, GType type)
-{
-  GTypeQuery query;
-
-  g_return_if_fail (SWFDEC_IS_AS_NATIVE_FUNCTION (function));
-  g_return_if_fail (g_type_is_a (type, SWFDEC_TYPE_AS_OBJECT));
-
-  g_type_query (type, &query);
-  function->type = type;
-}
-
 /**
  * swfdec_as_native_function_set_construct_type:
  * @function: a #SwfdecAsNativeFunction
@@ -318,63 +329,43 @@ swfdec_as_native_function_checkv (SwfdecAsContext *cx, SwfdecAsObject *object,
       return FALSE;
     *result = object;
   }
-  for (i = 0; *args; i++, args++) {
-    if (!optional && i >= argc && *args != '|')
-      break;
+  for (i = 0; *args && i < argc; i++, args++) {
     switch (*args) {
       case 'v':
 	{
 	  SwfdecAsValue *val = va_arg (varargs, SwfdecAsValue *);
-	  if (i < argc)
-	    *val = argv[i];
-	  else
-	    SWFDEC_AS_VALUE_SET_UNDEFINED (val);
+	  *val = argv[i];
 	}
 	break;
       case 'b':
 	{
 	  gboolean *b = va_arg (varargs, gboolean *);
-	  if (i < argc)
-	    *b = swfdec_as_value_to_boolean (cx, &argv[i]);
-	  else
-	    *b = FALSE;
+	  *b = swfdec_as_value_to_boolean (cx, &argv[i]);
 	}
 	break;
       case 'i':
 	{
 	  int *j = va_arg (varargs, int *);
-	  if (i < argc)
-	    *j = swfdec_as_value_to_integer (cx, &argv[i]);
-	  else
-	    *j = 0;
+	  *j = swfdec_as_value_to_integer (cx, &argv[i]);
 	}
 	break;
       case 'n':
 	{
 	  double *d = va_arg (varargs, double *);
-	  if (i < argc)
-	    *d = swfdec_as_value_to_number (cx, &argv[i]);
-	  else
-	    *d = 0;
+	  *d = swfdec_as_value_to_number (cx, &argv[i]);
 	}
 	break;
       case 's':
 	{
 	  const char **s = va_arg (varargs, const char **);
-	  if (i < argc)
-	    *s = swfdec_as_value_to_string (cx, &argv[i]);
-	  else
-	    *s = SWFDEC_AS_STR_EMPTY;
+	  *s = swfdec_as_value_to_string (cx, &argv[i]);
 	}
 	break;
       case 'o':
       case 'O':
 	{
 	  SwfdecAsObject **o = va_arg (varargs, SwfdecAsObject **);
-	  if (i < argc)
-	    *o = swfdec_as_value_to_object (cx, &argv[i]);
-	  else
-	    *o = NULL;
+	  *o = swfdec_as_value_to_object (cx, &argv[i]);
 	  if (*o == NULL && *args != 'O')
 	    return FALSE;
 	}
@@ -389,7 +380,7 @@ swfdec_as_native_function_checkv (SwfdecAsContext *cx, SwfdecAsObject *object,
 	return FALSE;
     }
   }
-  if (*args)
+  if (*args && !optional && *args != '|')
     return FALSE;
   return TRUE;
 }
