@@ -489,25 +489,26 @@ swfdec_as_value_to_number (SwfdecAsContext *context, const SwfdecAsValue *value)
 	// FIXME: We should most likely copy Tamarin's code here (MathUtils.cpp)
 	s = SWFDEC_AS_VALUE_GET_STRING (&tmp);
 	if (s == SWFDEC_AS_STR_EMPTY)
-	  return NAN;
+	  return (context->version >= 5) ? NAN : 0.0;
 	if (context->version > 5 && s[0] == '0' &&
 	    (s[1] == 'x' || s[1] == 'X')) {
 	  d = g_ascii_strtoll (s + 2, &end, 16);
-	} else if (context->version > 5 && s[0] == '0' &&
-	    s[strspn (s, "01234567")] == '\0') {
+	} else if (context->version > 5 &&
+	    (s[0] == '0' || ((s[0] == '+' || s[0] == '-') && s[1] == '0')) &&
+	    s[strspn (s+1, "01234567")+1] == '\0') {
 	  d = g_ascii_strtoll (s, &end, 8);
 	} else {
 	  if (strpbrk (s, "xXiI") != NULL)
-	    return NAN;
+	    return (context->version >= 5) ? NAN : 0.0;
 	  d = g_ascii_strtod (s, &end);
 	}
-	if (*end == '\0')
+	if (*end == '\0' || context->version < 5)
 	  return d == -0.0 ? 0.0 : d;
 	else
 	  return NAN;
       }
     case SWFDEC_AS_TYPE_OBJECT:
-      return NAN;
+      return (context->version >= 5) ? NAN : 0.0;
     case SWFDEC_AS_TYPE_INT:
     default:
       g_assert_not_reached ();
@@ -606,7 +607,6 @@ swfdec_as_value_to_object (SwfdecAsContext *context, const SwfdecAsValue *value)
       !SWFDEC_IS_AS_FUNCTION (fun = (SwfdecAsFunction *) SWFDEC_AS_VALUE_GET_OBJECT (&val)))
     return NULL;
   swfdec_as_object_create (fun, 1, value, &val);
-  swfdec_as_context_run (context);
   if (SWFDEC_AS_VALUE_IS_OBJECT (&val)) {
     return SWFDEC_AS_VALUE_GET_OBJECT (&val);
   } else {
@@ -706,3 +706,15 @@ swfdec_as_value_to_twips (SwfdecAsContext *context, const SwfdecAsValue *val,
   return TRUE;
 }
 
+SwfdecAsObject *
+swfdec_as_value_get_object (const SwfdecAsValue *val)
+{
+  SwfdecAsObject *o = val->value.object;
+
+  if (SWFDEC_IS_MOVIE (o)) {
+    o = SWFDEC_AS_OBJECT (swfdec_movie_resolve (SWFDEC_MOVIE (o)));
+    if (o == NULL)
+      o = val->value.object;
+  }
+  return o;
+}
