@@ -86,6 +86,9 @@ swfdec_key_get_type (void)
   static GType etype = 0;
   if (etype == 0) {
     static const GEnumValue values[] = {
+      { SWFDEC_KEY_LEFT_MOUSE, "SWFDEC_KEY_LEFT_MOUSE", "left-mouse" },
+      { SWFDEC_KEY_RIGHT_MOUSE, "SWFDEC_KEY_RIGHT_MOUSE", "right-mouse" },
+      { SWFDEC_KEY_MIDDLE_MOUSE, "SWFDEC_KEY_MIDDLE_MOUSE", "middle-mouse" },
       { SWFDEC_KEY_BACKSPACE, "SWFDEC_KEY_BACKSPACE", "backspace" },
       { SWFDEC_KEY_TAB, "SWFDEC_KEY_TAB", "tab" },
       { SWFDEC_KEY_CLEAR, "SWFDEC_KEY_CLEAR", "clear" },
@@ -174,6 +177,7 @@ swfdec_key_get_type (void)
       { SWFDEC_KEY_F14, "SWFDEC_KEY_F14", "f14" },
       { SWFDEC_KEY_F15, "SWFDEC_KEY_F15", "f15" },
       { SWFDEC_KEY_NUM_LOCK, "SWFDEC_KEY_NUM_LOCK", "num-lock" },
+      { SWFDEC_KEY_SCROLL_LOCK, "SWFDEC_KEY_SCROLL_LOCK", "scroll-lock" },
       { SWFDEC_KEY_SEMICOLON, "SWFDEC_KEY_SEMICOLON", "semicolon" },
       { SWFDEC_KEY_EQUAL, "SWFDEC_KEY_EQUAL", "equal" },
       { SWFDEC_KEY_COMMA, "SWFDEC_KEY_COMMA", "comma" },
