@@ -28,8 +28,9 @@
 
 #include "swfdec_text_field_movie.h"
 #include "swfdec_as_context.h"
-#include "swfdec_as_strings.h"
+#include "swfdec_as_internal.h"
 #include "swfdec_as_interpret.h"
+#include "swfdec_as_strings.h"
 #include "swfdec_debug.h"
 #include "swfdec_internal.h"
 #include "swfdec_player_internal.h"
@@ -75,14 +76,14 @@ swfdec_text_field_movie_update_area (SwfdecTextFieldMovie *text)
   cairo_matrix_t *matrix, translate;
   double x, y;
 
-  if (swfdec_player_is_locked (SWFDEC_PLAYER (SWFDEC_AS_OBJECT (text)->context)))
+  if (swfdec_player_is_locked (SWFDEC_PLAYER (swfdec_gc_object_get_context (text))))
     swfdec_movie_invalidate_next (movie);
 
   /* check if we indeed want to render */
   matrix = &text->to_layout;
   swfdec_movie_local_to_global_matrix (movie, matrix);
   cairo_matrix_multiply (matrix, matrix,
-      &SWFDEC_PLAYER (SWFDEC_AS_OBJECT (movie)->context)->priv->global_to_stage);
+      &SWFDEC_PLAYER (swfdec_gc_object_get_context (movie))->priv->global_to_stage);
   if (matrix->xy != 0.0 || matrix->yx != 0.0 ||
       matrix->xx <= 0.0 || matrix->yy <= 0.0) {
     swfdec_rectangle_init_empty (&text->stage_area);
@@ -205,26 +206,27 @@ swfdec_text_field_movie_invalidate (SwfdecMovie *movie, const cairo_matrix_t *ma
   }
 
   swfdec_rect_transform (&rect, &rect,
-      &SWFDEC_PLAYER (SWFDEC_AS_OBJECT (text)->context)->priv->stage_to_global);
+      &SWFDEC_PLAYER (swfdec_gc_object_get_context (text))->priv->stage_to_global);
   swfdec_player_invalidate (
-      SWFDEC_PLAYER (SWFDEC_AS_OBJECT (movie)->context), &rect);
+      SWFDEC_PLAYER (swfdec_gc_object_get_context (movie)), &rect);
 }
 
 static gboolean
 swfdec_text_field_movie_has_focus (SwfdecTextFieldMovie *text)
 {
-  SwfdecPlayer *player = SWFDEC_PLAYER (SWFDEC_AS_OBJECT (text)->context);
+  SwfdecPlayer *player = SWFDEC_PLAYER (swfdec_gc_object_get_context (text));
 
   return swfdec_player_has_focus (player, SWFDEC_ACTOR (text));
 }
 
 static void
 swfdec_text_field_movie_render (SwfdecMovie *movie, cairo_t *cr,
-    const SwfdecColorTransform *ctrans, const SwfdecRect *inval)
+    const SwfdecColorTransform *ctrans)
 {
   SwfdecTextFieldMovie *text = SWFDEC_TEXT_FIELD_MOVIE (movie);
   SwfdecRectangle *area;
   SwfdecColor color;
+  SwfdecRect inval;
 
   /* textfields don't mask */
   if (swfdec_color_transform_is_mask (ctrans) ||
@@ -263,8 +265,13 @@ swfdec_text_field_movie_render (SwfdecMovie *movie, cairo_t *cr,
     color = 0;
   }
 
-  /* render the layout */
   area = &text->layout_area;
+  /* Don't draw if out of clip */
+  cairo_clip_extents (cr, &inval.x0, &inval.y0, &inval.x1, &inval.y1);
+  if (inval.x1 <= area->x || inval.x0 >= area->x + area->width ||
+      inval.y1 <= area->y || inval.y0 >= area->y + area->height)
+    return;
+  /* render the layout */
   cairo_rectangle (cr, area->x, area->y, area->width, area->height);
   cairo_clip (cr);
   /* FIXME: This -1 is spacing? */
@@ -280,8 +287,6 @@ swfdec_text_field_movie_dispose (GObject *object)
 
   text = SWFDEC_TEXT_FIELD_MOVIE (object);
 
-  swfdec_text_attributes_reset (&text->default_attributes);
-
   if (text->style_sheet) {
     if (SWFDEC_IS_STYLESHEET (text->style_sheet)) {
       g_signal_handlers_disconnect_matched (text->style_sheet, 
@@ -305,7 +310,7 @@ swfdec_text_field_movie_dispose (GObject *object)
 }
 
 static void
-swfdec_text_field_movie_mark (SwfdecAsObject *object)
+swfdec_text_field_movie_mark (SwfdecGcObject *object)
 {
   SwfdecTextFieldMovie *text;
 
@@ -316,13 +321,13 @@ swfdec_text_field_movie_mark (SwfdecAsObject *object)
 
   swfdec_text_buffer_mark (text->text);
   if (text->style_sheet != NULL)
-    swfdec_as_object_mark (text->style_sheet);
+    swfdec_gc_object_mark (text->style_sheet);
   if (text->style_sheet_input != NULL)
     swfdec_as_string_mark (text->style_sheet_input);
   if (text->restrict_ != NULL)
     swfdec_as_string_mark (text->restrict_);
 
-  SWFDEC_AS_OBJECT_CLASS (swfdec_text_field_movie_parent_class)->mark (object);
+  SWFDEC_GC_OBJECT_CLASS (swfdec_text_field_movie_parent_class)->mark (object);
 }
 
 void
@@ -331,7 +336,7 @@ swfdec_text_field_movie_emit_onScroller (SwfdecTextFieldMovie *text)
   g_return_if_fail (SWFDEC_IS_TEXT_FIELD_MOVIE (text));
 
   if (!text->onScroller_emitted && swfdec_movie_get_version (SWFDEC_MOVIE (text)) > 6) {
-    swfdec_player_add_action (SWFDEC_PLAYER (SWFDEC_AS_OBJECT (text)->context), 
+    swfdec_player_add_action (SWFDEC_PLAYER (swfdec_gc_object_get_context (text)), 
 	SWFDEC_ACTOR (text), SWFDEC_EVENT_SCROLL, 0, SWFDEC_PLAYER_ACTION_QUEUE_NORMAL);
   }
   text->onScroller_emitted = TRUE;
@@ -389,20 +394,15 @@ swfdec_text_field_movie_init_movie (SwfdecMovie *movie)
 
   needs_unuse = swfdec_sandbox_try_use (movie->resource->sandbox);
 
-  cx = SWFDEC_AS_OBJECT (movie)->context;
+  cx = swfdec_gc_object_get_context (movie);
 
   swfdec_text_field_movie_init_properties (cx);
 
-  swfdec_as_object_get_variable (cx->global, SWFDEC_AS_STR_TextField, &val);
-  if (SWFDEC_AS_VALUE_IS_OBJECT (&val)) {
-    swfdec_as_object_set_constructor (SWFDEC_AS_OBJECT (movie),
-	SWFDEC_AS_VALUE_GET_OBJECT (&val));
-  }
+  swfdec_as_object_set_constructor_by_name (SWFDEC_AS_OBJECT (movie),
+      SWFDEC_AS_STR_TextField, NULL);
 
   /* create _listeners array */
   array = swfdec_as_array_new (cx);
-  if (array == NULL)
-    return;
   SWFDEC_AS_VALUE_SET_OBJECT (&val, SWFDEC_AS_OBJECT (movie));
   swfdec_as_array_push (SWFDEC_AS_ARRAY (array), &val);
   SWFDEC_AS_VALUE_SET_OBJECT (&val, array);
@@ -418,17 +418,30 @@ swfdec_text_field_movie_init_movie (SwfdecMovie *movie)
   text->background_color = SWFDEC_COLOR_COMBINE (255, 255, 255, 0);
 
   if (text_field) {
-    text->default_attributes.color = text_field->color;
-    text->default_attributes.align = text_field->align;
+    text->extents = SWFDEC_GRAPHIC (text_field)->extents;
+
+    text->html = text_field->html;
+    text->editable = text_field->editable;
+    swfdec_text_layout_set_password (text->layout, text_field->password);
+    text->max_chars = text_field->max_chars;
+    text->selectable = text_field->selectable;
+    text->embed_fonts = text_field->embed_fonts;
+    swfdec_text_layout_set_word_wrap (text->layout, text_field->word_wrap);
+    text->multiline = text_field->multiline;
+    text->auto_size = text_field->auto_size;
+    text->border = text_field->border;
+
+    text->text->default_attributes.color = text_field->color;
+    text->text->default_attributes.align = text_field->align;
     if (text_field->font != NULL)  {
-      text->default_attributes.font =
+      text->text->default_attributes.font =
 	swfdec_as_context_get_string (cx, text_field->font);
     }
-    text->default_attributes.size = text_field->size / 20;
-    text->default_attributes.left_margin = text_field->left_margin / 20;
-    text->default_attributes.right_margin = text_field->right_margin / 20;
-    text->default_attributes.indent = text_field->indent / 20;
-    text->default_attributes.leading = text_field->leading / 20;
+    text->text->default_attributes.size = text_field->size / 20;
+    text->text->default_attributes.left_margin = text_field->left_margin / 20;
+    text->text->default_attributes.right_margin = text_field->right_margin / 20;
+    text->text->default_attributes.indent = text_field->indent / 20;
+    text->text->default_attributes.leading = text_field->leading / 20;
   }
 
   // text
@@ -486,13 +499,13 @@ swfdec_text_field_movie_iterate (SwfdecActor *actor)
   SwfdecTextFieldMovie *text = SWFDEC_TEXT_FIELD_MOVIE (actor);
 
   while (text->changed) {
-    swfdec_player_add_action (SWFDEC_PLAYER (SWFDEC_AS_OBJECT (text)->context), 
+    swfdec_player_add_action (SWFDEC_PLAYER (swfdec_gc_object_get_context (text)), 
 	SWFDEC_ACTOR (text), SWFDEC_EVENT_CHANGED, 0, SWFDEC_PLAYER_ACTION_QUEUE_NORMAL);
     text->changed--;
   }
 
   if (text->onScroller_emitted && swfdec_movie_get_version (SWFDEC_MOVIE (text)) <= 6) {
-    swfdec_player_add_action (SWFDEC_PLAYER (SWFDEC_AS_OBJECT (text)->context), 
+    swfdec_player_add_action (SWFDEC_PLAYER (swfdec_gc_object_get_context (text)), 
 	SWFDEC_ACTOR (text), SWFDEC_EVENT_SCROLL, 0, SWFDEC_PLAYER_ACTION_QUEUE_NORMAL);
   }
   text->onScroller_emitted = FALSE;
@@ -514,6 +527,90 @@ swfdec_text_field_movie_contains (SwfdecMovie *movie, double x, double y,
   return movie;
 }
 
+static void
+swfdec_text_field_movie_parse_listen_variable (SwfdecTextFieldMovie *text,
+    const char *variable, SwfdecAsObject **object, const char **name)
+{
+  SwfdecAsContext *cx;
+  SwfdecAsObject *parent;
+  const char *p1, *p2;
+
+  g_return_if_fail (SWFDEC_IS_TEXT_FIELD_MOVIE (text));
+  g_return_if_fail (variable != NULL);
+  g_return_if_fail (object != NULL);
+  g_return_if_fail (name != NULL);
+
+  *object = NULL;
+  *name = NULL;
+
+  if (SWFDEC_MOVIE (text)->parent == NULL)
+    return;
+
+  g_assert (SWFDEC_IS_AS_OBJECT (SWFDEC_MOVIE (text)->parent));
+  cx = swfdec_gc_object_get_context (text);
+  parent = SWFDEC_AS_OBJECT (SWFDEC_MOVIE (text)->parent);
+
+  p1 = strrchr (variable, '.');
+  p2 = strrchr (variable, ':');
+  if (p1 == NULL && p2 == NULL) {
+    *object = parent;
+    *name = variable;
+  } else {
+    if (p1 == NULL || (p2 != NULL && p2 > p1))
+      p1 = p2;
+    if (strlen (p1) == 1)
+      return;
+    *object = swfdec_action_lookup_object (cx, parent, variable, p1);
+    if (*object == NULL)
+      return;
+    *name = swfdec_as_context_get_string (cx, p1 + 1);
+  }
+}
+
+static void
+swfdec_text_field_movie_asfunction (SwfdecTextFieldMovie *text,
+    const char *url)
+{
+  char **parts;
+  SwfdecAsObject *object;
+  const char *name;
+  SwfdecAsContext *cx;
+
+  g_return_if_fail (g_ascii_strncasecmp (url, "asfunction:",
+	strlen ("asfunction:")) == 0);
+
+  cx = swfdec_gc_object_get_context (text);
+
+  parts = g_strsplit (url + strlen ("asfunction:"), ",", 2);
+  if (parts[0] == NULL) {
+    SWFDEC_ERROR ("asfunction link without function name clicked");
+    g_strfreev (parts);
+    return;
+  }
+
+  swfdec_text_field_movie_parse_listen_variable (text,
+      swfdec_as_context_get_string (cx, parts[0]), &object, &name);
+
+  if (object == NULL || name == NULL) {
+    SWFDEC_ERROR ("Function in asfunction link not found: %s", parts[0]);
+    g_strfreev (parts);
+    return;
+  }
+
+  swfdec_sandbox_use (SWFDEC_MOVIE (text)->resource->sandbox);
+  if (parts[1] != NULL) {
+    SwfdecAsValue val;
+    SWFDEC_AS_VALUE_SET_STRING (&val,
+	swfdec_as_context_get_string (cx, parts[1]));
+    swfdec_as_object_call (object, name, 1, &val, NULL);
+  } else {
+    swfdec_as_object_call (object, name, 0, NULL, NULL);
+  }
+  swfdec_sandbox_use (SWFDEC_MOVIE (text)->resource->sandbox);
+
+  g_strfreev (parts);
+}
+
 static void
 swfdec_text_field_movie_letter_clicked (SwfdecTextFieldMovie *text,
     guint index_)
@@ -526,8 +623,13 @@ swfdec_text_field_movie_letter_clicked (SwfdecTextFieldMovie *text,
   attr = swfdec_text_buffer_get_attributes (text->text, index_);
 
   if (attr->url != SWFDEC_AS_STR_EMPTY) {
-    swfdec_player_launch (SWFDEC_PLAYER (SWFDEC_AS_OBJECT (text)->context),
-	attr->url, attr->target, NULL);
+    if (g_ascii_strncasecmp (attr->url, "asfunction:",
+	  strlen ("asfunction:")) == 0) {
+      swfdec_text_field_movie_asfunction (text, attr->url);
+    } else {
+      swfdec_player_launch (SWFDEC_PLAYER (swfdec_gc_object_get_context (text)),
+	  attr->url, attr->target, NULL);
+    }
   }
 }
 
@@ -606,7 +708,7 @@ swfdec_text_field_movie_mouse_press (SwfdecActor *actor, guint button)
     text->character_pressed = -1;
   }
 
-  swfdec_player_grab_focus (SWFDEC_PLAYER (SWFDEC_AS_OBJECT (text)->context), actor);
+  swfdec_player_grab_focus (SWFDEC_PLAYER (swfdec_gc_object_get_context (text)), actor);
 
   text->mouse_pressed = TRUE;
   swfdec_text_buffer_set_cursor (text->text, index_, index_);
@@ -787,12 +889,12 @@ swfdec_text_field_movie_property_set (SwfdecMovie *movie, guint prop_id,
     const SwfdecAsValue *val)
 {
   SwfdecTextFieldMovie *text = SWFDEC_TEXT_FIELD_MOVIE (movie);
-  SwfdecAsContext *cx = SWFDEC_AS_OBJECT (movie)->context;
+  SwfdecAsContext *cx = swfdec_gc_object_get_context (movie);
   SwfdecTwips twips;
 
   switch (prop_id) {
     case SWFDEC_MOVIE_PROPERTY_X:
-      if (!swfdec_as_value_to_twips (SWFDEC_AS_OBJECT (movie)->context, val, FALSE, &twips))
+      if (!swfdec_as_value_to_twips (swfdec_gc_object_get_context (movie), val, FALSE, &twips))
 	return;
       movie->modified = TRUE;
       twips -= text->extents.x0;
@@ -803,7 +905,7 @@ swfdec_text_field_movie_property_set (SwfdecMovie *movie, guint prop_id,
       }
       return;
     case SWFDEC_MOVIE_PROPERTY_Y:
-      if (!swfdec_as_value_to_twips (SWFDEC_AS_OBJECT (movie)->context, val, FALSE, &twips))
+      if (!swfdec_as_value_to_twips (swfdec_gc_object_get_context (movie), val, FALSE, &twips))
 	return;
       movie->modified = TRUE;
       twips -= text->extents.y0;
@@ -847,13 +949,13 @@ static void
 swfdec_text_field_movie_class_init (SwfdecTextFieldMovieClass * g_class)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (g_class);
-  SwfdecAsObjectClass *asobject_class = SWFDEC_AS_OBJECT_CLASS (g_class);
+  SwfdecGcObjectClass *gc_class = SWFDEC_GC_OBJECT_CLASS (g_class);
   SwfdecMovieClass *movie_class = SWFDEC_MOVIE_CLASS (g_class);
   SwfdecActorClass *actor_class = SWFDEC_ACTOR_CLASS (g_class);
 
   object_class->dispose = swfdec_text_field_movie_dispose;
 
-  asobject_class->mark = swfdec_text_field_movie_mark;
+  gc_class->mark = swfdec_text_field_movie_mark;
 
   movie_class->init_movie = swfdec_text_field_movie_init_movie;
   movie_class->finish_movie = swfdec_text_field_movie_finish_movie;
@@ -906,48 +1008,6 @@ swfdec_text_field_movie_init (SwfdecTextFieldMovie *text)
 
   text->mouse_wheel_enabled = TRUE;
   text->character_pressed = -1;
-
-  swfdec_text_attributes_reset (&text->default_attributes);
-}
-
-static void
-swfdec_text_field_movie_parse_listen_variable (SwfdecTextFieldMovie *text,
-    const char *variable, SwfdecAsObject **object, const char **name)
-{
-  SwfdecAsContext *cx;
-  SwfdecAsObject *parent;
-  const char *p1, *p2;
-
-  g_return_if_fail (SWFDEC_IS_TEXT_FIELD_MOVIE (text));
-  g_return_if_fail (variable != NULL);
-  g_return_if_fail (object != NULL);
-  g_return_if_fail (name != NULL);
-
-  *object = NULL;
-  *name = NULL;
-
-  if (SWFDEC_MOVIE (text)->parent == NULL)
-    return;
-
-  g_assert (SWFDEC_IS_AS_OBJECT (SWFDEC_MOVIE (text)->parent));
-  cx = SWFDEC_AS_OBJECT (text)->context;
-  parent = SWFDEC_AS_OBJECT (SWFDEC_MOVIE (text)->parent);
-
-  p1 = strrchr (variable, '.');
-  p2 = strrchr (variable, ':');
-  if (p1 == NULL && p2 == NULL) {
-    *object = parent;
-    *name = variable;
-  } else {
-    if (p1 == NULL || (p2 != NULL && p2 > p1))
-      p1 = p2;
-    if (strlen (p1) == 1)
-      return;
-    *object = swfdec_action_lookup_object (cx, parent, variable, p1);
-    if (*object == NULL)
-      return;
-    *name = swfdec_as_context_get_string (cx, p1 + 1);
-  }
 }
 
 void
@@ -980,7 +1040,7 @@ swfdec_text_field_movie_variable_listener_callback (SwfdecAsObject *object,
 
   text = SWFDEC_TEXT_FIELD_MOVIE (object);
   swfdec_text_field_movie_set_text (text,
-      swfdec_as_value_to_string (object->context, val), text->html);
+      swfdec_as_value_to_string (swfdec_gc_object_get_context (text), val), text->html);
 }
 
 void
@@ -990,10 +1050,6 @@ swfdec_text_field_movie_set_listen_variable (SwfdecTextFieldMovie *text,
   SwfdecAsObject *object;
   const char *name;
 
-  // FIXME: case-insensitive when v < 7?
-  if (text->variable == value)
-    return;
-
   if (text->variable != NULL) {
     swfdec_text_field_movie_parse_listen_variable (text, text->variable,
 	&object, &name);
@@ -1007,14 +1063,23 @@ swfdec_text_field_movie_set_listen_variable (SwfdecTextFieldMovie *text,
   text->variable = value;
 
   if (value != NULL) {
+    SwfdecTextField *text_field =
+      SWFDEC_TEXT_FIELD (SWFDEC_MOVIE (text)->graphic);
     SwfdecAsValue val;
 
     swfdec_text_field_movie_parse_listen_variable (text, value, &object,
 	&name);
     if (object != NULL && swfdec_as_object_get_variable (object, name, &val)) {
       swfdec_text_field_movie_set_text (text,
-	  swfdec_as_value_to_string (SWFDEC_AS_OBJECT (text)->context, &val),
+	  swfdec_as_value_to_string (swfdec_gc_object_get_context (text), &val),
 	  text->html);
+    } else if (text_field != NULL && text_field->input != NULL) {
+      // Set to the original value from the tag, not current value
+      const char *initial = swfdec_as_context_get_string (
+	  swfdec_gc_object_get_context (text), text_field->input);
+      swfdec_text_field_movie_set_listen_variable_text (text, initial);
+      // FIXME: html value correct here?
+      swfdec_text_field_movie_set_text (text, initial, text_field->html);
     }
     if (object != NULL && SWFDEC_IS_MOVIE (object)) {
       swfdec_movie_add_variable_listener (SWFDEC_MOVIE (object),
@@ -1053,39 +1118,7 @@ swfdec_text_field_movie_get_text (SwfdecTextFieldMovie *text)
     *p = '\r';
   }
 
-  return swfdec_as_context_give_string (SWFDEC_AS_OBJECT (text)->context, ret);
-}
-
-void
-swfdec_text_field_movie_replace_text (SwfdecTextFieldMovie *text,
-    guint start_index, guint end_index, const char *str)
-{
-  g_return_if_fail (SWFDEC_IS_TEXT_FIELD_MOVIE (text));
-  g_return_if_fail (end_index <= swfdec_text_buffer_get_length (text->text));
-  g_return_if_fail (start_index <= end_index);
-  g_return_if_fail (str != NULL);
-
-  /* if there was a style sheet set when setting the text, modifications are
-   * not allowed */
-  if (text->style_sheet_input)
-    return;
-
-  if (end_index > start_index)
-    swfdec_text_buffer_delete_text (text->text, start_index, end_index - start_index);
-
-  if (start_index == swfdec_text_buffer_get_length (text->text) &&
-      start_index > 0) {
-    const SwfdecTextAttributes *attr = swfdec_text_buffer_get_attributes (text->text, 0);
-    if (SWFDEC_AS_OBJECT (text)->context->version < 8) {
-      SWFDEC_FIXME ("replaceText to the end of the TextField might use wrong text format on version 7");
-    }
-    swfdec_text_buffer_insert_text (text->text, start_index, str);
-    swfdec_text_buffer_set_attributes (text->text, start_index, 
-	swfdec_text_buffer_get_length (text->text) - start_index, attr, 
-	SWFDEC_TEXT_ATTRIBUTES_MASK);
-  } else {
-    swfdec_text_buffer_insert_text (text->text, start_index, str);
-  }
+  return swfdec_as_context_give_string (swfdec_gc_object_get_context (text), ret);
 }
 
 void
@@ -1098,16 +1131,14 @@ swfdec_text_field_movie_set_text (SwfdecTextFieldMovie *text, const char *str,
   g_return_if_fail (str != NULL);
 
   /* Flash 7 resets to default style here */
-  if (html && SWFDEC_AS_OBJECT (text)->context->version < 8)
-    swfdec_text_attributes_reset (&text->default_attributes);
+  if (html && swfdec_gc_object_get_context (text)->version < 8)
+    swfdec_text_buffer_reset_default_attributes (text->text);
 
   length = swfdec_text_buffer_get_length (text->text);
   if (length)
     swfdec_text_buffer_delete_text (text->text, 0, length);
-  swfdec_text_buffer_set_attributes (text->text, 0, 0, &text->default_attributes,
-      SWFDEC_TEXT_ATTRIBUTES_MASK);
 
-  if (SWFDEC_AS_OBJECT (text)->context->version >= 7 &&
+  if (swfdec_gc_object_get_context (text)->version >= 7 &&
       text->style_sheet != NULL)
   {
     text->style_sheet_input = str;
