@@ -303,6 +303,9 @@ swfdec_text_field_movie_set_restrict (SwfdecAsContext *cx,
 
   SWFDEC_AS_CHECK (SWFDEC_TYPE_TEXT_FIELD_MOVIE, &text, "s", &value);
 
+  if (text->restrict_ == NULL && value == SWFDEC_AS_STR_EMPTY)
+    return;
+
   if (SWFDEC_AS_VALUE_IS_UNDEFINED (&argv[0]) ||
       SWFDEC_AS_VALUE_IS_NULL (&argv[0])) {
     text->restrict_ = NULL;
@@ -492,9 +495,9 @@ swfdec_text_field_movie_get_backgroundColor (SwfdecAsContext *cx,
   SWFDEC_AS_CHECK (SWFDEC_TYPE_TEXT_FIELD_MOVIE, &text, "");
 
   SWFDEC_AS_VALUE_SET_NUMBER (ret,
-      SWFDEC_COLOR_R (text->background_color) << 16 |
-      SWFDEC_COLOR_G (text->background_color) << 8 |
-      SWFDEC_COLOR_B (text->background_color));
+      SWFDEC_COLOR_RED (text->background_color) << 16 |
+      SWFDEC_COLOR_GREEN (text->background_color) << 8 |
+      SWFDEC_COLOR_BLUE (text->background_color));
 }
 
 static void
@@ -556,9 +559,9 @@ swfdec_text_field_movie_get_borderColor (SwfdecAsContext *cx,
 
 
   SWFDEC_AS_VALUE_SET_NUMBER (ret,
-      SWFDEC_COLOR_R (text->border_color) << 16 |
-      SWFDEC_COLOR_G (text->border_color) << 8 |
-      SWFDEC_COLOR_B (text->border_color));
+      SWFDEC_COLOR_RED (text->border_color) << 16 |
+      SWFDEC_COLOR_GREEN (text->border_color) << 8 |
+      SWFDEC_COLOR_BLUE (text->border_color));
 }
 
 static void
@@ -742,7 +745,6 @@ swfdec_text_field_movie_set_autoSize (SwfdecAsContext *cx,
     SwfdecAsValue *ret)
 {
   SwfdecTextFieldMovie *text;
-  SwfdecAutoSize old;
   const char *s;
 
   SWFDEC_AS_CHECK (SWFDEC_TYPE_TEXT_FIELD_MOVIE, &text, "");
@@ -750,7 +752,6 @@ swfdec_text_field_movie_set_autoSize (SwfdecAsContext *cx,
   if (argc < 1)
     return;
 
-  old = text->auto_size;
   if (SWFDEC_AS_VALUE_IS_BOOLEAN (&argv[0])) {
     if (SWFDEC_AS_VALUE_GET_BOOLEAN (&argv[0])) {
       text->auto_size = SWFDEC_AUTO_SIZE_LEFT;
@@ -761,14 +762,14 @@ swfdec_text_field_movie_set_autoSize (SwfdecAsContext *cx,
     swfdec_as_value_to_number (cx, &argv[0]);
     s = swfdec_as_value_to_string (cx, &argv[0]);
 
-    if (!g_ascii_strcasecmp (s, "none")) {
-      text->auto_size = SWFDEC_AUTO_SIZE_NONE;
-    } else if (!g_ascii_strcasecmp (s, "left")) {
+    if (!g_ascii_strcasecmp (s, "left")) {
       text->auto_size = SWFDEC_AUTO_SIZE_LEFT;
     } else if (!g_ascii_strcasecmp (s, "right")) {
       text->auto_size = SWFDEC_AUTO_SIZE_RIGHT;
     } else if (!g_ascii_strcasecmp (s, "center")) {
       text->auto_size = SWFDEC_AUTO_SIZE_CENTER;
+    } else {
+      text->auto_size = SWFDEC_AUTO_SIZE_NONE;
     }
   }
 }
@@ -941,7 +942,8 @@ swfdec_text_field_movie_get_textColor (SwfdecAsContext *cx,
 
   SWFDEC_AS_CHECK (SWFDEC_TYPE_TEXT_FIELD_MOVIE, &text, "");
 
-  SWFDEC_AS_VALUE_SET_NUMBER (ret, text->default_attributes.color);
+  SWFDEC_AS_VALUE_SET_NUMBER (ret, 
+      swfdec_text_buffer_get_default_attributes (text->text)->color);
 }
 
 // This doesn't work the same way as TextFormat's color setting
@@ -955,7 +957,8 @@ swfdec_text_field_movie_set_textColor (SwfdecAsContext *cx,
 
   SWFDEC_AS_CHECK (SWFDEC_TYPE_TEXT_FIELD_MOVIE, &text, "i", &value);
 
-  text->default_attributes.color = swfdec_text_field_movie_int_to_color (cx, value);
+  /* FIXME: don't access private variables, you lazy hacker! */
+  text->text->default_attributes.color = swfdec_text_field_movie_int_to_color (cx, value);
 }
 
 SWFDEC_AS_NATIVE (104, 300, swfdec_text_field_movie_get_gridFitType)
@@ -1066,7 +1069,8 @@ swfdec_text_field_movie_getNewTextFormat (SwfdecAsContext *cx,
   if (format == NULL)
     return;
 
-  swfdec_text_attributes_copy (&format->attr, &text->default_attributes,
+  swfdec_text_attributes_copy (&format->attr, 
+      swfdec_text_buffer_get_default_attributes (text->text),
       SWFDEC_TEXT_ATTRIBUTES_MASK);
   format->values_set = SWFDEC_TEXT_ATTRIBUTES_MASK;
 
@@ -1087,8 +1091,8 @@ swfdec_text_field_movie_setNewTextFormat (SwfdecAsContext *cx,
   if (!SWFDEC_IS_TEXT_FORMAT (format))
     return;
 
-  swfdec_text_attributes_copy (&text->default_attributes, &format->attr,
-      format->values_set);
+  swfdec_text_buffer_set_default_attributes (text->text, 
+      &format->attr, format->values_set);
 }
 
 SWFDEC_AS_NATIVE (104, 102, swfdec_text_field_movie_setTextFormat)
@@ -1239,6 +1243,10 @@ swfdec_text_field_movie_replaceText (SwfdecAsContext *cx,
     return;
   if (end_index < start_index)
     return;
+  /* if there was a style sheet set when setting the text, modifications are
+   * not allowed */
+  if (text->style_sheet_input)
+    return;
 
   string = swfdec_text_buffer_get_text (text->text);
   start = end = g_utf8_strlen (string, -1);
@@ -1247,7 +1255,15 @@ swfdec_text_field_movie_replaceText (SwfdecAsContext *cx,
   start = g_utf8_offset_to_pointer (string, start) - string;
   end = g_utf8_offset_to_pointer (string, end) - string;
 
-  swfdec_text_field_movie_replace_text (text, start, end, str);
+  /* anyone explain to me why Flash 7 has this special case plz */
+  if (end > start && (str != SWFDEC_AS_STR_EMPTY || cx->version != 7))
+    swfdec_text_buffer_delete_text (text->text, start, end - start);
+
+  swfdec_text_buffer_insert_text (text->text, start, str);
+  /* yes, modifying text changes the default format */
+  swfdec_text_buffer_set_default_attributes (text->text,
+      swfdec_text_buffer_get_attributes (text->text, 0),
+      SWFDEC_TEXT_ATTRIBUTES_MASK);
 }
 
 // static
@@ -1328,6 +1344,8 @@ swfdec_text_field_movie_createTextField (SwfdecAsContext *cx,
   SwfdecAsValue val;
 
   SWFDEC_AS_CHECK (SWFDEC_TYPE_MOVIE, &parent, "siiiii", &name, &depth, &x, &y, &width, &height);
+  width = ABS (width);
+  height = ABS (height);
 
   edittext = g_object_new (SWFDEC_TYPE_TEXT_FIELD, NULL);
   edittext->html = FALSE;
@@ -1396,10 +1414,8 @@ swfdec_text_field_movie_createTextField (SwfdecAsContext *cx,
       SWFDEC_AS_STR___constructor__, &val,
       SWFDEC_AS_VARIABLE_HIDDEN | SWFDEC_AS_VARIABLE_VERSION_6_UP);
 
-  swfdec_as_function_call (fun, SWFDEC_AS_OBJECT (movie), 0, NULL, 
-      cx->version > 7 ? rval : NULL);
-  cx->frame->construct = TRUE;
-  swfdec_as_context_run (cx);
+  swfdec_as_function_call_full (fun, SWFDEC_AS_OBJECT (movie), TRUE, NULL,
+      0, NULL, cx->version > 7 ? rval : &val);
 }
 
 void
@@ -1534,18 +1550,8 @@ swfdec_text_field_movie_construct (SwfdecAsContext *cx, SwfdecAsObject *object,
     guint argc, SwfdecAsValue *argv, SwfdecAsValue *ret)
 {
   if (!cx->frame->construct) {
-    SwfdecAsValue val;
-    if (!swfdec_as_context_use_mem (cx, sizeof (SwfdecAsObject)))
-      return;
-    object = g_object_new (SWFDEC_TYPE_AS_OBJECT, NULL);
-    swfdec_as_object_add (object, cx, sizeof (SwfdecAsObject));
-    swfdec_as_object_get_variable (cx->global, SWFDEC_AS_STR_TextField, &val);
-    if (SWFDEC_AS_VALUE_IS_OBJECT (&val)) {
-      swfdec_as_object_set_constructor (object,
-	  SWFDEC_AS_VALUE_GET_OBJECT (&val));
-    } else {
-      SWFDEC_INFO ("\"TextField\" is not an object");
-    }
+    object = swfdec_as_object_new_empty (cx);
+    swfdec_as_object_set_constructor_by_name (object, SWFDEC_AS_STR_TextField, NULL);
   }
 
   swfdec_text_field_movie_init_properties (cx);
