@@ -40,6 +40,7 @@ typedef enum {
   REWRITE_TRACE_FUNCTION_NAME = (1 << 0),
   REWRITE_RANDOM              = (1 << 1),
   REWRITE_GETTERS	      = (1 << 2),
+  REWRITE_ENUMERATE           = (1 << 3),
 
   REWRITE_INIT		      =	(1 << 17)
 } RewriteOptions;
@@ -164,6 +165,84 @@ insert_function_trace (ViviCodeAssembler *assembler, const char *name)
   }
 }
 
+static void
+sort_enumerate (ViviCodeAssembler *assembler)
+{
+  guint i, j;
+
+  for (i = 0; i < vivi_code_assembler_get_n_codes (assembler); i++) {
+    ViviCodeAsm *code = vivi_code_assembler_get_code (assembler, i);
+    if (VIVI_IS_CODE_ASM_ENUMERATE (code) ||
+	VIVI_IS_CODE_ASM_ENUMERATE2 (code)) {
+      ViviCodeStatement *statement;
+      ViviCodeCompiler *compiler;
+      ViviCodeAssembler *asm2;
+
+      // FIXME: compile once and use multiple times (problem is the labels)
+      statement = vivi_parse_string (
+	  "asm {"
+	  "  push reg 0;"
+	  "  push '$_vivi_rewrite_sort_enumerate';"
+	  "  define_local;"
+
+	  "  push 0;"
+	  "  init_array;"
+	  "  store 0;"
+	  "  pop;"
+
+	  "read_start:"
+	  "  push_duplicate;"
+	  "  push undefined;"
+	  "  equals2;"
+	  "  if read_end;"
+
+	  "  push 1, reg 0, 'push';"
+	  "  call_method;"
+	  "  pop;"
+	  "  jump read_start;"
+
+	  "read_end:"
+	  "  pop;"
+
+	  "  push 0, reg 0, 'sort';"
+	  "  call_method;"
+	  "  pop;"
+
+	  "  push undefined;"
+	  "write_start:"
+	  "  push 0, reg 0, 'length';"
+	  "  get_member;"
+	  "  equals2;"
+	  "  if write_end;"
+
+	  "  push 0, reg 0, 'pop';"
+	  "  call_method;"
+	  "  jump write_start;"
+
+	  "write_end:"
+	  "  push '$_vivi_rewrite_sort_enumerate';"
+	  "  get_variable;"
+	  "  store 0;"
+	  "  pop;"
+	  "  push '$_vivi_rewrite_sort_enumerate';"
+	  "  delete2;"
+	  "}"
+	  );
+      g_assert (statement);
+      compiler = vivi_code_compiler_new (7); // FIXME: version
+      vivi_code_compiler_compile_statement (compiler, statement);
+      g_object_unref (statement);
+      asm2 = g_object_ref (vivi_code_compiler_get_assembler (compiler));
+      g_object_unref (compiler);
+      for (j = 0; j < vivi_code_assembler_get_n_codes (asm2); j++) {
+	vivi_code_assembler_insert_code (assembler, ++i,
+	    vivi_code_assembler_get_code (asm2, j));
+      }
+      g_object_unref (asm2);
+    }
+  }
+}
+
 static void
 replace_random (ViviCodeAssembler *assembler, guint init)
 {
@@ -211,14 +290,21 @@ replace_random (ViviCodeAssembler *assembler, guint init)
 static void
 rewrite_getters (ViviCodeAssembler *assembler)
 {
-  guint i;
+  guint i, count;
+  char *s;
 
+  count = 0;
   for (i = 0; i < vivi_code_assembler_get_n_codes (assembler); i++) {
     ViviCodeAsm *code = vivi_code_assembler_get_code (assembler, i);
     if (VIVI_IS_CODE_ASM_GET_VARIABLE (code) ||
 	VIVI_IS_CODE_ASM_GET_MEMBER (code)) {
       i++;
       INSERT_CODE (assembler, i, vivi_code_asm_push_duplicate_new ());
+      s = g_strdup_printf ("%u: ", ++count);
+      INSERT_PUSH_STRING (assembler, i, s);
+      g_free (s);
+      INSERT_CODE (assembler, i, vivi_code_asm_swap_new ());
+      INSERT_CODE (assembler, i, vivi_code_asm_add2_new ());
       INSERT_CODE (assembler, i, vivi_code_asm_trace_new ());
       i--;
     }
@@ -245,6 +331,8 @@ do_script (SwfdecBuffer *buffer, guint flags, const char *name, guint version)
     replace_random (assembler, flags & REWRITE_INIT);
   if (flags & REWRITE_GETTERS)
     rewrite_getters (assembler);
+  if (flags & REWRITE_ENUMERATE)
+    sort_enumerate (assembler);
 
   script = vivi_code_assembler_assemble_script (assembler, version, &error);
   g_object_unref (assembler);
@@ -402,11 +490,13 @@ main (int argc, char *argv[])
   gboolean trace_function_names = FALSE;
   gboolean random = FALSE;
   gboolean getters = FALSE;
+  gboolean enumerate = FALSE;
 
   GOptionEntry options[] = {
     { "getters", 'g', 0, G_OPTION_ARG_NONE, &getters, "trace all variable get operations", NULL },
     { "trace-function-names", 'n', 0, G_OPTION_ARG_NONE, &trace_function_names, "trace names of called functions", NULL },
     { "no-random", 'r', 0, G_OPTION_ARG_NONE, &random, "replace all random values with 0", NULL },
+    { "sort-enumerate", 'e', 0, G_OPTION_ARG_NONE, &enumerate, "sort all enumerate actions to have predictable order", NULL },
     { NULL }
   };
   GOptionContext *ctx;
@@ -444,7 +534,8 @@ main (int argc, char *argv[])
   buffer = process_buffer (buffer,
       (trace_function_names ? REWRITE_TRACE_FUNCTION_NAME : 0) |
       (random ? REWRITE_RANDOM : 0) |
-      (getters ? REWRITE_GETTERS : 0));
+      (getters ? REWRITE_GETTERS : 0) |
+      (enumerate ? REWRITE_ENUMERATE : 0));
   if (buffer == NULL) {
     g_printerr ("\"%s\": Broken Flash file\n", argv[1]);
     return 1;
