@@ -31,6 +31,7 @@
 #include "vivi_code_and.h"
 #include "vivi_code_asm_code_default.h"
 #include "vivi_code_asm_get_url2.h"
+#include "vivi_code_asm_if.h"
 #include "vivi_code_asm_jump.h"
 #include "vivi_code_asm_pool.h"
 #include "vivi_code_asm_push.h"
@@ -389,14 +390,14 @@ vivi_parser_get_waiting_label (ParseData *data, const char *name)
   for (iter = data->level->waiting_labels; iter != NULL; iter = iter->next) {
     ViviCodeLabel *label = VIVI_CODE_LABEL (iter->data);
     if (g_str_equal (vivi_code_label_get_name (label), name))
-      return label;
+      return g_object_ref (label);
   }
 
   return NULL;
 }
 
 static void
-vivi_parser_remove_waiting_label (ParseData *data, const ViviCodeLabel *label)
+vivi_parser_remove_waiting_label (ParseData *data, ViviCodeLabel *label)
 {
   g_return_if_fail (data != NULL);
   g_return_if_fail (data->level != NULL);
@@ -404,6 +405,7 @@ vivi_parser_remove_waiting_label (ParseData *data, const ViviCodeLabel *label)
 
   data->level->waiting_labels =
     g_slist_remove (data->level->waiting_labels, label);
+  g_object_unref (label);
 }
 
 static ViviCodeLabel *
@@ -418,7 +420,7 @@ vivi_parser_get_label (ParseData *data, const char *name)
   for (iter = data->level->labels; iter != NULL; iter = iter->next) {
     ViviCodeLabel *label = VIVI_CODE_LABEL (iter->data);
     if (g_str_equal (vivi_code_label_get_name (label), name))
-      return label;
+      return g_object_ref (label);
   }
 
   return vivi_parser_get_waiting_label (data, name);
@@ -1011,6 +1013,28 @@ parse_asm_store (ParseData *data)
   return vivi_code_asm_store_new (id);
 }
 
+static ViviCodeAsm *
+parse_asm_if (ParseData *data)
+{
+  ViviCodeAsm *code;
+  ViviCodeLabel *label;
+  const char *name;
+
+  name = parse_identifier_value (data);
+  label = vivi_parser_get_label (data, name);
+  if (label == NULL) {
+    label = VIVI_CODE_LABEL (vivi_code_label_new (name));
+    vivi_parser_add_waiting_label (data, label);
+  }
+
+  parse_automatic_semicolon (data);
+
+  code = vivi_code_asm_if_new (label);
+  g_object_unref (label);
+
+  return code;
+}
+
 static ViviCodeAsm *
 parse_asm_jump (ParseData *data)
 {
@@ -1298,6 +1322,7 @@ static const AsmStatement asm_statements[] = {
 #include "vivi_code_defaults.h"
 #undef DEFAULT_ASM
   { "get_url2", NULL, parse_asm_get_url2 },
+  { "if", NULL, parse_asm_if },
   { "jump", NULL, parse_asm_jump },
   { "pool", NULL, parse_asm_pool },
   { "push", NULL, parse_asm_push },
@@ -1336,6 +1361,8 @@ parse_asm_code (ParseData *data)
     identifier = g_strdup ("throw");
   } else if (try_parse_token (data, TOKEN_TRY)) {
     identifier = g_strdup ("try");
+  } else if (try_parse_token (data, TOKEN_IF)) {
+    identifier = g_strdup ("if");
   } else if (try_parse_token (data, TOKEN_IMPLEMENTS)) {
     identifier = g_strdup ("implements");
   } else if (try_parse_token (data, TOKEN_DELETE)) {
@@ -1395,7 +1422,7 @@ peek_asm_statement (ParseData *data)
 
   value = vivi_parser_scanner_get_value (data->scanner, 2);
 
-  return (value->token == TOKEN_BRACE_LEFT);
+  return (value->token == TOKEN_BRACE_LEFT || value->token == TOKEN_PARENTHESIS_LEFT);
 }
 
 static ViviCodeStatement *
@@ -1408,6 +1435,27 @@ parse_asm_statement (ParseData *data)
 
   if (g_ascii_strcasecmp (parse_identifier_value (data), "asm") != 0)
     vivi_parser_error (data, "Expected 'asm'");
+
+  if (try_parse_token (data, TOKEN_PARENTHESIS_LEFT)) {
+    ViviCodeValue **arguments;
+    guint i;
+
+    if (!try_parse_token (data, TOKEN_PARENTHESIS_RIGHT)) {
+      parse_value_list (data, peek_assignment_expression,
+	  parse_assignment_expression, &arguments, TOKEN_COMMA);
+      parse_token (data, TOKEN_PARENTHESIS_RIGHT);
+    } else {
+      arguments = NULL;
+    }
+
+    if (arguments != NULL) {
+      for (i = 0; arguments[i] != NULL; i++) {
+	vivi_code_assembler_push_argument (assembler, arguments[i]);
+      }
+      free_value_list (arguments);
+    }
+  }
+
   parse_token (data, TOKEN_BRACE_LEFT);
 
   if (!try_parse_token (data, TOKEN_BRACE_RIGHT)) {
@@ -1418,6 +1466,7 @@ parse_asm_statement (ParseData *data)
     } while (peek_token (data, TOKEN_IDENTIFIER) ||
 	peek_token (data, TOKEN_THROW) ||
 	peek_token (data, TOKEN_TRY) ||
+	peek_token (data, TOKEN_IF) ||
 	peek_token (data, TOKEN_IMPLEMENTS) ||
 	peek_token (data, TOKEN_DELETE) ||
 	peek_token (data, TOKEN_RETURN) ||
